@@ -6,11 +6,11 @@
     @license: GNU GPL, see COPYING for details.
 """
 
-import StringIO, os, re, urllib, random, codecs
+import StringIO, os, re, random, codecs
 
 from MoinMoin import config, caching, user, util, wikiutil
 from MoinMoin.logfile import eventlog
-from MoinMoin.util import filesys, web
+from MoinMoin.util import filesys, MoinMoinNoFooter, timefuncs
 
 class Page:
     """Page - Manage an (immutable) page associated with a WikiName.
@@ -46,6 +46,7 @@ class Page:
         #print >>sys.stderr, "page %s" % repr(page_name)
         #traceback.print_stack(limit=4, file=sys.stderr)
 
+
         if keywords.has_key('formatter'):
             self.formatter = keywords.get('formatter')
             self.default_formatter = 0
@@ -56,6 +57,7 @@ class Page:
         self._raw_body_modified = 0
         self.hilite_re = None
         self.language = None
+        self.pi_format = None
 
         self.reset()
 
@@ -120,13 +122,17 @@ class Page:
         return rev
 
     def get_rev_dir(self, pagedir, rev=0):
-        """
-        get a revision of a page from an arbitrary pagedir.
-        does not modify page object's state, uncached, direct disk access.
-        @param pagedir: the path to the pagedir
+        """Get a revision of a page from an arbitrary pagedir.
+        
+        Does not modify page object's state, uncached, direct disk
+        access.
+
+        @param pagedir: the path to the page storage area
         @param rev: int revision to get (default is 0 and means the current
                     revision (in this case, the real revint is returned)
-        @return: (str pagefilename, int realrevint, bool exists)
+        @return: (str path to file of the revision,
+                  int realrevint,
+                  bool exists)
         """
         if rev == 0:
             rev = self.get_current_from_pagedir(pagedir)
@@ -134,16 +140,48 @@ class Page:
         revstr = '%08d' % rev
         pagefile = os.path.join(pagedir, 'revisions', revstr)
         exists = os.path.exists(pagefile)
+        if exists:
+            self._setRealPageName(pagedir)
         return pagefile, rev, exists
 
-    def get_rev(self, use_underlay=-1, rev=0):
+    def _setRealPageName(self, pagedir):
+        """ Set page_name to the real case of page name
+        
+        On case insensitive file system, "pagename" exists even if the
+        real page name is "PageName" or "PAGENAME". This leads to
+        consufion in urls, links and logs.
+        See MoinMoinBugs/MacHfsPlusCaseInsensitive
+        
+        Correct the case of the page name. Elements created from the
+        page name in reset() are not updated because its too messy, and
+        this fix seems to be enough for 1.3.
+        
+        Problems to fix later:
+        
+         - ["helponnavigation"] link to HelpOnNavigation but not
+           considered as backlink.
+        
+        @param pagedir: the storage path to the page directory
         """
-        get a revision of this page
+        realPath = filesys.realPathCase(pagedir)
+        if realPath is None:
+            return
+        realPath = wikiutil.unquoteWikiname(realPath)
+        self.page_name = realPath[-len(self.page_name):]
+
+    def get_rev(self, use_underlay=-1, rev=0):
+        """Get information about a revision.
+
+        filename, number, and (existance test) of this page and revision.
+
         @param use_underlay: -1 == auto, 0 == normal, 1 == underlay
         @param rev: int revision to get (default is 0 and means the current
                     revision (in this case, the real revint is returned)
-        @return: (str pagefilename, int realrevint, bool exists)
+        @return: (str path to current revision of page,
+                  int realrevint,
+                  bool exists)
         """
+        # Figure out if we should use underlay or not, if needed.
         if use_underlay == -1:
             if self._underlay is not None and self._pagepath[self._underlay] is not None:
                 underlay = self._underlay
@@ -153,6 +191,7 @@ class Page:
         else:
             underlay, pagedir = use_underlay, self._pagepath[use_underlay]
 
+        # Find current revision, if automatic selection is requested.
         if rev == 0:
             if self._current_rev[underlay] is None:
                 realrev = self.get_current_from_pagedir(pagedir)
@@ -160,6 +199,8 @@ class Page:
             else:
                 realrev = self._current_rev[underlay]
 
+            # This restores from cache.  If they are None (uncached),
+            # they will be generated at the very end.
             _exists = self._exists[underlay]
             _realrev = self._current_rev[underlay]
             _pagefile = self._pagefile[underlay]
@@ -178,6 +219,12 @@ class Page:
         return pagefile, realrev, exists
 
     def current_rev(self):
+        """Return number of current revision.
+        
+        This is the same as get_rev()[1].
+        
+        @return: int revision
+        """
         pagefile, rev, exists = self.get_rev()
         return rev
 
@@ -202,7 +249,8 @@ class Page:
                                 '1' = use underlay page dir
                                 '0' = use standard page dir
         @rtype: string
-        @return: the full path to the storage area
+        @return: int underlay,
+                 str the full path to the storage area
         """
         standardpath, underlaypath = self._pagepath
         if underlaypath is None:
@@ -272,7 +320,8 @@ class Page:
                                (default true)
         @keyword isfile: is the last component in args a filename? (default is false)
         @rtype: string
-        @return: the full path to the storage area
+        @return: (int underlay (1 if using underlay, 0 otherwise),
+                  str the full path to the storage area )
         """
         check_create = kw.get('check_create', 1)
         isfile = kw.get('isfile', 0)
@@ -289,6 +338,8 @@ class Page:
         return underlay, fullpath
 
     def getPagePath(self, *args, **kw):
+        """Return path to the page storage area."""
+
         return self.getPageStatus(*args, **kw)[1]
 
     def split_title(self, request, force=0):
@@ -365,10 +416,12 @@ class Page:
         if not self.rev:
             log = self._last_edited(request)
             if log:
-                editordata = log.getEditorData(request)
+                editordata = log.getInterwikiEditorData(request)
                 editor = editordata[1]
-                if editordata[0] == 'homepage':
-                    editor = editordata[1].page_name
+                if editordata[0] == 'interwiki':
+                    editor = "%s:%s" % editordata[1]
+                else: # 'ip'
+                    editor = editordata[1]
                 result = {
                     'timestamp': log.ed_time_usecs,
                     'editor': editor,
@@ -529,7 +582,8 @@ class Page:
 
     def mtime_printable(self, request):
         """
-        Get printable modification timestamp of this page.
+        Get printable (as per user's preferences) modification
+        timestamp of this page.
 
         @rtype: string
         @return: formatted string with mtime of page
@@ -551,7 +605,7 @@ class Page:
         If you want to get a more accurate number, call with
         exists=1. This will be about 100 times slower though.
 
-        @param exits: filter existing pages
+        @param exists: filter existing pages
         @rtype: int
         @return: number of pages
         """
@@ -569,7 +623,7 @@ class Page:
         return count
 
     def getPageList(self, user=None, exists=1, filter=None):
-        ''' List user readable pages under current page
+        """ List user readable pages under current page
 
         Currently only request.rootpage is used to list pages, but if we
         have true sub pages, any page can list its sub pages.
@@ -593,10 +647,10 @@ class Page:
 
         @param user: the user requesting the pages (MoinMoin.user.User)
         @param filter: filter function
-        @param exits: filter existing pages
+        @param exists: filter existing pages
         @rtype: list of unicode strings
         @return: user readable wiki page names
-        '''
+        """
         request = self.request
         request.clock.start('getPageList')
         # Check input
@@ -648,7 +702,7 @@ class Page:
 
         @param user: the user requesting the pages
         @param filter: filter function
-        @param exits: only existing pages
+        @param exists: only existing pages
         @rtype: dict {unicode: Page}
         @return: user readable pages
         """
@@ -706,14 +760,17 @@ class Page:
         return pages
 
     def getlines(self):
+        """ Return a list of all lines in body.
+
+        @rtype: list
+        @return: list of strs body_lines"""
         lines = self.get_raw_body().split('\n')
         return lines
 
     def get_raw_body(self):
-        """
-        Load the raw markup from the page file.
+        """ Load the raw markup from the page file.
 
-        @rtype: string
+        @rtype: str
         @return: raw page contents of this page
         """
         if self._raw_body is None:
@@ -747,8 +804,8 @@ class Page:
 
         @param body: raw body text
         @param modified: 1 means that we internally modified the raw text and
-                         that it is not in sync with the page file on disk.
-                         This is used e.g. by PageEditor when previewing the page.
+            that it is not in sync with the page file on disk.  This is
+            used e.g. by PageEditor when previewing the page.
         """
         self._raw_body = body
         self._raw_body_modified = modified
@@ -758,8 +815,8 @@ class Page:
 
         @param request: the request object
         @param querystr: the query string to add after a "?" after the url
-            (str or dict, see util.web.makeQueryString)
-        @param escpae: escape url for html, to be backward compatible
+            (str or dict, see wikiutil.makeQueryString)
+        @param escape: escape url for html, to be backward compatible
             with old code (bool)
         @rtype: str
         @return: complete url of this page, including scriptname
@@ -768,14 +825,14 @@ class Page:
                      wikiutil.quoteWikinameURL(self.page_name))
 
         if querystr:
-            querystr = web.makeQueryString(querystr)
+            querystr = wikiutil.makeQueryString(querystr)
 
-            # TODO: remove in 1.4
+            # TODO: remove in 2.0
             # Escape query string to be compatible with old 3rd party code
             # New code should call with escape=0 to prevent the warning.
             if escape:
                 import warnings
-                warnings.warn("In moin 1.4 query string in url will not be"
+                warnings.warn("In moin 2.0 query string in url will not be"
                               " escaped. See"
                               " http://moinmoin.wikiwikiweb.de/ApiChanges")
                 querystr = wikiutil.escape(querystr)
@@ -805,14 +862,16 @@ class Page:
         # Create url, excluding scriptname
         url = wikiutil.quoteWikinameURL(self.page_name)
         if querystr:
-            querystr = web.makeQueryString(querystr)
-            # makeQueryString does not escape any more
-            querystr = wikiutil.escape(querystr)
+            if not isinstance(querystr, type({})):
+                # makeQueryString does not escape strings any more
+                querystr = wikiutil.escape(querystr)
+                
+            querystr = wikiutil.makeQueryString(querystr)
             url = "%s?%s" % (url, querystr)
 
         # Add anchor
         if anchor:
-            url = "%s#%s" % (url, urllib.quote_plus(anchor.encode(config.charset)))
+            url = "%s#%s" % (url, wikiutil.url_quote_plus(anchor))
 
         # Add css class for non existing page
         if not self.exists():
@@ -820,7 +879,6 @@ class Page:
 
         link = wikiutil.link_tag(request, url, wikiutil.escape(text),
                                  formatter=getattr(self, 'formatter', None), **kw)
-
         # Create a link to attachments if any exist
         if kw.get('attachment_indicator', 0):
             from MoinMoin.action import AttachFile
@@ -874,8 +932,9 @@ class Page:
             if not UserPerms(subscriber).read(self.page_name): continue
 
             if subscriber.isSubscribedTo(pageList):
-                lang = subscriber.language or 'en'
-                if not subscriber_list.has_key(lang): subscriber_list[lang] = []
+                lang = subscriber.language or request.cfg.language_default
+                if not lang in subscriber_list:
+                    subscriber_list[lang] = []
                 if return_users:
                     subscriber_list[lang].append(subscriber)
                 else:
@@ -884,9 +943,21 @@ class Page:
         return subscriber_list
 
 
+    def send_raw(self):
+        """ Output the raw page data (action=raw) """
+        self.request.http_headers(["Content-type: text/plain;charset=%s" % config.charset])
+        text = self.get_raw_body()
+        text = self.encodeTextMimeType(text)
+        self.request.write(text)
+        raise MoinMoinNoFooter
+
+
     def send_page(self, request, msg=None, **keywords):
-        """
-        Output the formatted page.
+        """ Output the formatted page.
+
+        TODO: remove request argument - page is created with a request
+        instance. Enable removing request argument recursively from all
+        functions called from here.
 
         @param request: the request object
         @param msg: if given, display message in header area
@@ -894,6 +965,7 @@ class Page:
         @keyword content_id: set the id of the enclosing div
         @keyword count_hit: if 1, add an event to the log
         @keyword send_missing_page: if 1, assume that page to be sent is MissingPage
+        @keyword omit_footnotes: if True, do not send footnotes (used by include macro)
         """
         from MoinMoin import i18n
         request.clock.start('send_page')
@@ -906,13 +978,12 @@ class Page:
         else:
             media = 'screen'
         content_only = keywords.get('content_only', 0)
+        omit_footnotes = keywords.get('omit_footnotes', 0)
         content_id = keywords.get('content_id', 'content')
         do_cache = keywords.get('do_cache', 1)
         send_missing_page = keywords.get('send_missing_page', 0)
-        if request.form.has_key('highlight'):
-            self.hilite_re = request.form['highlight'][0]
-        else:
-            self.hilite_re = None
+        self.hilite_re = (keywords.get('hilite_re') or
+                          request.form.get('highlight', [None])[0])
         if msg is None: msg = ""
 
         # count hit?
@@ -937,7 +1008,7 @@ class Page:
         pi_refresh = None
         pi_formtext = []
         pi_formfields = []
-        wikiform = None
+        pi_lines = 0
 
         # check for XML content
         if body and body[:5] == '<?xml':
@@ -945,6 +1016,8 @@ class Page:
 
         # check processing instructions
         while body and body[0] == '#':
+            pi_lines += 1
+            
             # extract first line
             try:
                 line, body = body.split('\n', 1)
@@ -1003,14 +1076,14 @@ class Page:
                 request.http_redirect('%s/%s?action=show&redirect=%s' % (
                     request.getScriptname(),
                     wikiutil.quoteWikinameURL(pi_redirect),
-                    urllib.quote_plus(self.page_name.encode(config.charset), ''),))
+                    wikiutil.url_quote_plus(self.page_name, ''),))
                 return
             elif verb == "deprecated":
                 # deprecated page, append last backup version to current contents
                 # (which should be a short reason why the page is deprecated)
                 msg = '%s<strong>%s</strong><br>%s' % (
                     self.formatter.smiley('/!\\'),
-                    _('The backupped content of this page is deprecated and will not be included in search results!'),
+                    _('The backed up content of this page is deprecated and will not be included in search results!'),
                     msg)
 
                 revisions = self.getRevList()
@@ -1026,19 +1099,6 @@ class Page:
                     pass
                 else:
                     request.setPragma(key, val)
-            elif verb == "form":
-                # ignore form PIs on non-form pages
-                if not wikiutil.isFormPage(request, self.page_name):
-                    continue
-
-                # collect form definitions
-                if not wikiform:
-                    from MoinMoin import wikiform
-                    # TODO: form probably can work with action=""
-                    pi_formtext.append('<table border="1" cellspacing="1" cellpadding="3">\n'
-                        '<form method="POST" action="%s">\n'
-                        '<input type="hidden" name="action" value="formtest">\n' % self.url(request))
-                pi_formtext.append(wikiform.parseDefinition(request, args, pi_formfields))
             elif verb == "acl":
                 # We could build it here, but there's no request.
                 pass
@@ -1060,8 +1120,26 @@ class Page:
         page_exists = self.exists()
         if not content_only:
             # send the document leader
+
+            # need to inform caches that content changes
+            # based on cookie (even if we aren't sending one now)
+            request.setHttpHeader("Vary: Cookie")
+            # we include User-Agent because a bot might be denied and get no content
+            request.setHttpHeader("Vary: User-Agent")
+
+            # use "nocache" headers if we're using a method that
+            # is not simply "display", or if a user is logged in
+            # (which triggers personalisation features)
+
             if page_exists:
-                request.http_headers()
+                if not request.cacheable or request.user.valid:
+                    request.http_headers(request.nocache)
+                else:
+                    # use the correct last-modified value from the on-disk file
+                    # to ensure cacheability where supported
+                    request.http_headers(["Last-Modified: " +
+                         timefuncs.formathttpdate(os.path.getmtime(self._text_filename()))])
+
             else:
                 request.http_headers(['Status: 404 NOTFOUND'])
                 request.setResponseCode(404)
@@ -1073,7 +1151,7 @@ class Page:
                 link = '%s/%s?action=fullsearch&amp;value=%s&amp;context=180' % (
                     request.getScriptname(),
                     wikiutil.quoteWikinameURL(self.page_name),
-                    urllib.quote_plus(full_text_query.encode(config.charset)))
+                    wikiutil.url_quote_plus(full_text_query))
 
                 title = self.split_title(request)
                 if self.rev:
@@ -1123,13 +1201,14 @@ class Page:
                     ) % {'pagename': self.formatter.text(self.page_name)})
                     request.write(''.join(pi_formtext))
 
-        # try to load the parser
-        Parser = wikiutil.importPlugin(self.request.cfg, "parser",
-                                       self.pi_format, "Parser")
-        if Parser is None:
-            # default to plain text formatter (i.e. show the page source)
-            del Parser
-            from parser.plain import Parser
+        # Load the parser, or default to plain text parser that will
+        # just show the page raw source.
+        # TODO: do we need this magic? any effect on debugging?
+        try:
+            Parser = wikiutil.importPlugin(self.request.cfg, "parser", 
+                                           self.pi_format, "Parser")
+        except wikiutil.PluginMissingError:
+            from MoinMoin.parser.plain import Parser
 
         # start wiki content div
         request.write(self.formatter.startContent(content_id))
@@ -1142,7 +1221,7 @@ class Page:
             elif content_only and send_missing_page:
                 # We should send MissingPage but it is not there
                 import warnings
-                warnings.warn("Error - The page MissingPage could not be found."
+                warnings.warn("Error - The page MissingPage or MissingHomePage could not be found."
                               " Check your underlay directory setting.")
                 url = '%s?action=edit' % wikiutil.quoteWikinameURL(self.page_name)
                 request.write(wikiutil.link_tag(self.request, url, text=_("Create New Page"),
@@ -1151,10 +1230,13 @@ class Page:
             request.write("<strong>%s</strong><br>" % _("You are not allowed to view this page."))
         else:
             # parse the text and send the page content
-            self.send_page_content(request, Parser, body, format_args=pi_formatargs, do_cache=do_cache)
+            self.send_page_content(request, Parser, body,
+                                   format_args=pi_formatargs,
+                                   do_cache=do_cache,
+                                   start_line=pi_lines)
 
             # check for pending footnotes
-            if getattr(request, 'footnotes', None):
+            if getattr(request, 'footnotes', None) and not omit_footnotes:
                 from MoinMoin.macro.FootNote import emit_footnotes
                 request.write(emit_footnotes(request, self.formatter))
 
@@ -1180,17 +1262,15 @@ class Page:
         request.clock.stop('send_page')
 
     def getFormatterName(self):
-        """ Return a formatter name, used in the caching system
+        """ Return a formatter name as used in the caching system
 
         @rtype: string
         @return: formatter name as used in caching
         """
         if not hasattr(self, 'formatter'):
             return ''
-
-        name = str(self.formatter.__class__)
-        name = name.replace('MoinMoin.formatter.', '').replace('.Formatter', '')
-        return name
+        module = self.formatter.__module__
+        return module[module.rfind('.') + 1:]
 
     def canUseCache(self, parser=None):
         """ Is caching available for this request?
@@ -1209,137 +1289,109 @@ class Page:
             not self._raw_body_modified and
             self.getFormatterName() in self.cfg.caching_formats):
             # Everything is fine, now check the parser:
-            if not parser:
-                parser = wikiutil.importPlugin(self.request.cfg, "parser",
-                                               self.pi_format, "Parser")
+            if parser is None:
+                try:
+                    parser = wikiutil.importPlugin(self.request.cfg, "parser",
+                                                   self.pi_format, "Parser")
+                except wikiutil.PluginMissingError:
+                    pass
             return getattr(parser, 'caching', False)
-
         return False
 
-    def send_page_content(self, request, Parser, body, needsupdate=0,
-                          format_args='', do_cache=1):
-        # XXX CFG remove request param? self.request ?
-        """
-        Output the formatted wiki page, using caching, if possible.
+    def send_page_content(self, request, Parser, body, format_args='',
+                          do_cache=1, **kw):
+        """ Output the formatted wiki page, using caching if possible
 
         @param request: the request object
-        @param Parser: the Parser
+        @param Parser: Parser class
         @param body: text of the wiki page
-        @param needsupdate: if 1, force update of the cached compiled page
+        @param format_args: #format arguments, used by some parsers
+        @param do_cache: if True, use cached content
         """
         request.clock.start('send_page_content')
+        parser = Parser(body, request, format_args=format_args, **kw)
 
-        formatter_name = self.getFormatterName()
-
-        # if we should not or can not use caching
         if not (do_cache and self.canUseCache(Parser)):
-            # parse the text and send the page content
-            Parser(body, request, format_args=format_args).format(self.formatter)
-            request.clock.stop('send_page_content')
-            return
-
-        #try cache
-        _ = request.getText
-        from MoinMoin import wikimacro
-        arena = self
-        key = formatter_name
-        cache = caching.CacheEntry(request, arena, key)
-        code = None
-
-        if cache.needsUpdate(self._text_filename(),
-                             self.getPagePath('attachments', check_create=0)):
-            needsupdate = 1
-
-        # load cache
-        if not needsupdate:
+            self.format(parser)
+        else:
             try:
-                import marshal
-                code = marshal.loads(cache.content())
-            except (EOFError, ValueError, TypeError), err:
-                # Bad marshal data, must update the cache.
-                # See http://docs.python.org/lib/module-marshal.html
-                needsupdate = 1
-            except:
-                # Log unexpected errors
-                import sys
-                err = sys.exc_info()[1]
-                err = ('Unexpected error while trying to load "%(pagename)s"'
-                       ' cache: %(errorname)s: %(error)s\n') % {
-                    'pagename': self.page_name.encode(config.charset),
-                    'errorname': err.__class__.__name__,
-                    'error': str(err),
-                    }
-                request.log(err)
-                # And also update the cache
-                needsupdate = 1
-
-        # render page
-        if needsupdate:
-            # Save request state before - rendering page might
-            # change request state!
-            saved_current_lang = request.current_lang
-
-            from MoinMoin.formatter.text_python import Formatter
-            formatter = Formatter(request, ["page"], self.formatter)
-
-            import marshal
-            buffer = StringIO.StringIO()
-            request.redirect(buffer)
-            parser = Parser(body, request)
-            parser.format(formatter)
-            request.redirect()
-            text = buffer.getvalue()
-            buffer.close()
-            src = formatter.assemble_code(text)
-            ## request.write(src) # debug
-            code = compile(src.encode(config.charset),
-                           self.page_name.encode(config.charset), 'exec')
-            cache.update(marshal.dumps(code))
-
-            # Restore request state
-            request.current_lang = saved_current_lang
-
-        # send page
-        formatter = self.formatter
-        parser = Parser(body, request)
-        macro_obj = wikimacro.Macro(parser)
-
-        try:
-            # if this code module is in a .zip file, mess with __file__
-            # in order to get caching right
-            moinmodule = __import__('MoinMoin')
-            if hasattr(moinmodule, '__loader__'):
-                __file__ = os.path.join(moinmodule.__loader__.archive, 'dummy')
-
-            exec code
-        except 'CacheNeedsUpdate':
-            # if something goes wrong, try without caching
-            # Prevent wrong timing in recursive calls
-            request.clock.stop('send_page_content')
-            self.send_page_content(request, Parser, body, needsupdate=1)
-            request.clock.start('send_page_content')
-            cache = caching.CacheEntry(request, arena, key)
-
-        # Save my cache modification time, this info might be used by
-        # themes - but only after the page content was sent.
-        self.cache_mtime = cache.mtime()
-
-        # TODO: move this into theme (currently used only by classic)
-        url = wikiutil.quoteWikinameURL(self.page_name) + '?action=refresh'
-        link = wikiutil.link_tag(request, url, _("DeleteCache", formatted=False))
-        date = self.request.user.getFormattedDateTime(cache.mtime())
-        fragment = link + ' ' +  _('(cached %s)') % date
-        self.request.add2footer('DeleteCache', fragment)
-
+                code = self.loadCache(request)
+                self.execute(request, parser, code)
+            except 'CacheNeedsUpdate':
+                try:
+                    code = self.makeCache(request, parser)
+                    self.execute(request, parser, code)
+                except 'CacheNeedsUpdate':
+                    request.log('page cache failed after creation')
+                    self.format(parser)
+        
         request.clock.stop('send_page_content')
 
+    def format(self, parser):
+        """ Format and write page content without caching """
+        parser.format(self.formatter)
+
+    def execute(self, request, parser, code):
+        """ Write page content by executing cache code """            
+        formatter = self.formatter
+        from MoinMoin import wikimacro        
+        macro_obj = wikimacro.Macro(parser)        
+        # Fix __file__ when running from a zip package
+        import MoinMoin
+        if hasattr(MoinMoin, '__loader__'):
+            __file__ = os.path.join(MoinMoin.__loader__.archive, 'dummy')
+        exec code
+
+    def loadCache(self, request):
+        """ Return page content cache or raises 'CacheNeedsUpdate' """
+        cache = caching.CacheEntry(request, self, self.getFormatterName())
+        attachmentsPath = self.getPagePath('attachments', check_create=0)
+        if cache.needsUpdate(self._text_filename(), attachmentsPath):
+            raise 'CacheNeedsUpdate'
+        
+        import marshal
+        try:
+            return marshal.loads(cache.content())
+        except (EOFError, ValueError, TypeError):
+            # Bad marshal data, must update the cache.
+            # See http://docs.python.org/lib/module-marshal.html
+            raise 'CacheNeedsUpdate'
+        except Exception, err:
+            request.log('fail to load "%s" cache: %s' % 
+                        (self.page_name, str(err)))
+            raise 'CacheNeedsUpdate'
+
+    def makeCache(self, request, parser):
+        """ Format content into code, update cache and return code """
+        import marshal        
+        from MoinMoin.formatter.text_python import Formatter
+        formatter = Formatter(request, ["page"], self.formatter)
+        
+        # Save request state while formatting page
+        saved_current_lang = request.current_lang
+        try:
+            text = request.redirectedOutput(parser.format, formatter)
+        finally:
+            request.current_lang = saved_current_lang
+        
+        src = formatter.assemble_code(text)
+        code = compile(src.encode(config.charset),
+                       self.page_name.encode(config.charset), 'exec')
+        cache = caching.CacheEntry(request, self, self.getFormatterName())
+        cache.update(marshal.dumps(code))
+        self.cache_mtime = cache.mtime()
+        return code
+
     def _emptyPageText(self, request):
         """
         Output the default page content for new pages.
 
         @param request: the request object
         """
-        missingpage = wikiutil.getSysPage(request, 'MissingPage')
+        if request.user.valid and request.user.name == self.page_name:
+            missingpage = wikiutil.getSysPage(request, 'MissingHomePage')
+        else:
+            missingpage = wikiutil.getSysPage(request, 'MissingPage')
         missingpagefn = missingpage._text_filename()
         missingpage.page_name = self.page_name
         missingpage._text_filename_force = missingpagefn
@@ -1493,7 +1545,7 @@ class Page:
         @rtype: Page
         @return: parent page or None
         """
-        if config.allow_subpages and self.page_name:
+        if self.page_name:
             pos = self.page_name.rfind('/')
             if pos > 0:
                 parent = Page(self.request, self.page_name[:pos])
@@ -1511,9 +1563,6 @@ class Page:
         @return: ACL of this page
         """
         import wikiacl
-        # Return new empty acl when acl is not enabled
-        if not self.cfg.acl_enabled:
-            return wikiacl.AccessControlList(request)
         request.clock.start('getACL')
         # Try the cache or parse acl and update the cache
         currentRevision = self.current_rev()
