@@ -13,65 +13,24 @@ from MoinMoin.Page import Page
 from MoinMoin.widget import html
 from MoinMoin.widget.dialog import Status
 from MoinMoin.logfile import editlog, eventlog
-from MoinMoin.util import filesys
+from MoinMoin.util import filesys, timefuncs
 import MoinMoin.util.web
 import MoinMoin.util.mail
-import MoinMoin.util.datetime
 
 
 #############################################################################
 ### Javascript code for editor page
 #############################################################################
 
-# This code is internal to allow I18N, else we'd use a .js file;
 # we avoid the "--" operator to make this XHTML happy!
 _countdown_js = """
+%(countdown_script)s
 <script type="text/javascript">
-var timeout_min = %(lock_timeout)s;
-var state = 0; // 0: start; 1: long count; 2: short count; 3: timeout; 4/5: blink
-var counter = 0, step = 1, delay = 1;
-
-function countdown() {
-    // change state if counter is down
-    if (counter <= 1) {
-        state += 1
-        if (state == 1) {
-            counter = timeout_min
-            step = 1
-            delay = 60000
-        }
-        if (state == 2) {
-            counter = 60
-            step = 5
-            delay = step * 1000
-        }
-        if (state == 3 || state == 5) {
-            window.status = "%(lock_expire)s"
-            state = 3
-            counter = 1
-            step = 1
-            delay = 500
-        }
-        if (state == 4) {
-            // blink the above text
-            window.status = " "
-            counter = 1
-            delay = 250
-        }
-    }
-
-    // display changes
-    if (state < 3) {
-        var msg
-        if (state == 1) msg = "%(lock_mins)s"
-        if (state == 2) msg = "%(lock_secs)s"
-        window.status = msg.replace(/#/, counter)
-    }
-    counter -= step
-
-    // Set timer for next update
-    setTimeout("countdown()", delay)
-}
+var countdown_timeout_min = %(lock_timeout)s
+var countdown_lock_expire = "%(lock_expire)s"
+var countdown_lock_mins = "%(lock_mins)s"
+var countdown_lock_secs = "%(lock_secs)s"
+addLoadEvent(countdown)
 </script>
 """
 
@@ -152,6 +111,19 @@ class PageEditor(Page):
 
         return False
 
+    def sendconfirmleaving(self):
+        """ to prevent moving away from the page without saving it """
+        _ = self._
+        self.request.write(u'''\
+<script type="text/javascript">
+    var flgChange = false;
+    function confirmleaving() {
+        if ( flgChange )
+            return "%s";
+    }
+</script>
+''' % _("Your changes are not saved!"))
+        
     def sendEditor(self, **kw):
         """
         Send the editor form page.
@@ -170,11 +142,12 @@ class PageEditor(Page):
         _ = self._
         self.request.http_headers(self.request.nocache)
 
+        raw_body = ''
         msg = None
         conflict_msg = None
         edit_lock_message = None
         preview = kw.get('preview', None)
-        emit_anchor = not kw.get('staytop', 0)
+        staytop = kw.get('staytop', 0)
 
         from MoinMoin.formatter.text_html import Formatter
         self.request.formatter = Formatter(self.request, store_pagelinks=1)
@@ -195,7 +168,7 @@ class PageEditor(Page):
                 if not ok:
                     # failed to get the lock
                     if preview is not None:
-                        edit_lock_message = _('The lock you held timed out, be prepared for editing conflicts!'
+                        edit_lock_message = _('The lock you held timed out. Be prepared for editing conflicts!'
                             ) + "<br>" + edit_lock_message
                     else:
                         msg = edit_lock_message
@@ -274,21 +247,29 @@ Have a look at the diff of %(difflink)s to see what has been changed.""") % {
             title % {'pagename': self.split_title(self.request),},
             page=self,
             pagename=self.page_name, msg=status,
-            body_onload=self.lock.locktype and 'countdown()' or '', # broken / bug in Mozilla 1.5, when using #preview
             html_head=self.lock.locktype and (
                 _countdown_js % {
+                     'countdown_script': self.request.theme.externalScript('countdown'),
                      'lock_timeout': lock_timeout,
                      'lock_expire': lock_expire,
                      'lock_mins': lock_mins,
                      'lock_secs': lock_secs,
-                    }) or ''
+                    }) or '',
+            editor_mode=1,
         )
         
         self.request.write(self.request.formatter.startContent("content"))
-        
-        # get the text body for the editor field
-        if form.has_key('template'):
-            # "template" parameter contains the name of the template page
+
+        # Get the text body for the editor field.
+        # TODO: what about deleted pages? show the text of the last revision or use the template?
+        if preview is not None:
+            raw_body = self.get_raw_body()
+        elif self.exists():
+            # If the page exists, we get the text from the page.
+            # TODO: maybe warn if template argument was ignored because the page exists?
+            raw_body = self.get_raw_body()
+        elif form.has_key('template'):
+            # If the page does not exists, we try to get the content from the template parameter.
             template_page = wikiutil.unquoteWikiname(form['template'][0])
             if self.request.user.may.read(template_page):
                 raw_body = Page(self.request, template_page).get_raw_body()
@@ -297,42 +278,7 @@ Have a look at the diff of %(difflink)s to see what has been changed.""") % {
                 else:
                     self.request.write(_("[Template %s not found]") % (template_page,), '<br>')
             else:
-                raw_body = ''
                 self.request.write(_("[You may not read %s]") % (template_page,), '<br>')
-        else:
-            raw_body = self.get_raw_body()
-
-        # send text above text area
-        template_param = ''
-        if form.has_key('template'):
-            template_param = '&amp;template=' + form['template'][0]
-        self.request.write('<p>')
-        self.request.write(wikiutil.getSysPage(self.request, 'HelpOnFormatting').link_to(self.request))
-        self.request.write(" | ", wikiutil.getSysPage(self.request, 'InterWiki').link_to(self.request))
-        if preview is not None and emit_anchor:
-            self.request.write(' | <a href="#preview">%s</a>' % _('Skip to preview'))
-        self.request.write(' ')
-        self.request.write(_('[current page size \'\'\'%(size)d\'\'\' bytes]') % {'size': self.size()})
-        self.request.write('</p>')
-        
-        # send form
-        self.request.write('<form id="editor" method="post" action="%s/%s#preview">' % (
-            self.request.getScriptname(),
-            wikiutil.quoteWikinameURL(self.page_name),
-            ))
-
-        # yet another weird workaround for broken IE6 (it expands the text
-        # editor area to the right after you begin to type...). IE sucks...
-        # http://fplanque.net/2003/Articles/iecsstextarea/
-        self.request.write('<fieldset style="border:none;padding:0;">')
-        
-        self.request.write('<p>')
-        self.request.write(unicode(html.INPUT(type="hidden", name="action", value="savepage")))
-
-        # Save backto in a hidden input
-        backto = form.get('backto', [None])[0]
-        if backto:
-            self.request.write(unicode(html.INPUT(type="hidden", name="backto", value=backto)))
 
         # Make backup on previews - but not for new empty pages
         if preview and raw_body:
@@ -342,32 +288,88 @@ Have a look at the diff of %(difflink)s to see what has been changed.""") % {
         if not raw_body:
             raw_body = _('Describe %s here.') % (self.page_name,)
 
+        # send form
+        self.request.write('<form id="editor" method="post" action="%s/%s#preview" onSubmit="flgChange = false;">' % (
+            self.request.getScriptname(),
+            wikiutil.quoteWikinameURL(self.page_name),
+            ))
+
+        # yet another weird workaround for broken IE6 (it expands the text
+        # editor area to the right after you begin to type...). IE sucks...
+        # http://fplanque.net/2003/Articles/iecsstextarea/
+        self.request.write('<fieldset style="border:none;padding:0;">')
+        
+        self.request.write(unicode(html.INPUT(type="hidden", name="action", value="edit")))
+
         # Send revision of the page our edit is based on
         self.request.write('<input type="hidden" name="rev" value="%d">' % (rev,))
 
+        # Save backto in a hidden input
+        backto = form.get('backto', [None])[0]
+        if backto:
+            self.request.write(unicode(html.INPUT(type="hidden", name="backto", value=backto)))
+
+        # button bar
+        button_spellcheck = (SpellCheck and
+            '<input class="button" type="submit" name="button_spellcheck" value="%s" onClick="flgChange = false;">'
+                % _('Check Spelling')) or ''
+
+        save_button_text = _('Save Changes')
+        cancel_button_text = _('Cancel')
+        
+        if self.cfg.page_license_enabled:
+            self.request.write('<p><em>', _(
+"""By hitting '''%(save_button_text)s''' you put your changes under the %(license_link)s.
+If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.""") % {
+                'save_button_text': save_button_text,
+                'cancel_button_text': cancel_button_text,
+                     'license_link': wikiutil.getSysPage(self.request, self.cfg.page_license_page).link_to(self.request),
+            }, '</em></p>')
+
+
+        self.request.write('''
+<input class="button" type="submit" name="button_save" value="%s" onClick="flgChange = false;">
+<input class="button" type="submit" name="button_preview" value="%s" onClick="flgChange = false;">
+''' % (save_button_text, _('Preview'),))
+        
+        if not (self.request.cfg.editor_force and self.request.cfg.editor_default == 'text'):
+            self.request.write('''
+<input id="switch2gui" style="display: none;" class="button" type="submit" name="button_switch" value="%s">
+''' % (_('GUI Mode'),))
+            
+        self.request.write('''
+%s
+<input class="button" type="submit" name="button_cancel" value="%s">
+<input type="hidden" name="editor" value="text">
+''' % (button_spellcheck, cancel_button_text,))
+
         # Add textarea with page text
 
         # TODO: currently self.language is None at this point. We have
         # to do processing instructions parsing earlier, or move page
         # language into meta file.
-        lang = self.language or self.request.cfg.default_lang
+        lang = self.language or self.request.cfg.language_default
+
+        self.sendconfirmleaving()
 
         self.request.write(
-            u'<textarea id="editor-textarea" name="savetext" lang="%(lang)s" dir="%(dir)s"'
-            u' rows="%(rows)d">' % {
+            u'''\
+<textarea id="editor-textarea" name="savetext" lang="%(lang)s" dir="%(dir)s" rows="%(rows)d"
+          onChange="flgChange = true;" onKeyPress="flgChange = true;">\
+%(text)s\
+</textarea>''' %   {
             'lang': lang,
             'dir': i18n.getDirection(lang),
             'rows': text_rows,
-            }
-            )
-        self.request.write(wikiutil.escape(raw_body))
-        self.request.write(u'</textarea>')
+            'text': wikiutil.escape(raw_body)
+        })
 
-        self.request.write('</p>')
-
-        self.request.write("<p>", _("Optional comment about this change"),
-            '<br><input id="editor-comment" type="text" name="comment" value="%s" maxlength="80"></p>' % (
+        self.request.write("<p>")
+        self.request.write(_("Comment:"),
+            ' <input id="editor-comment" type="text" name="comment" value="%s" maxlength="200"'
+            ' onChange="flgChange = true;" onKeyPress="flgChange = true;">' % (
                 wikiutil.escape(kw.get('comment', ''), 1), ))
+        self.request.write("</p>")
 
         # Category selection
         filter = re.compile(self.cfg.page_category_regex, re.UNICODE).search
@@ -375,53 +377,29 @@ Have a look at the diff of %(difflink)s to see what has been changed.""") % {
         cat_pages.sort()
         cat_pages = [wikiutil.pagelinkmarkup(p) for p in cat_pages]
         cat_pages.insert(0, ('', _('<No addition>', formatted=False)))
-        self.request.write("<p>", _('Make this page belong to category %(category)s') % {
+        self.request.write("<p>")
+        self.request.write(_('Add to: %(category)s') % {
             'category': unicode(util.web.makeSelection('category', cat_pages)),
         })
 
-        # button bar
-        button_spellcheck = (SpellCheck and
-            '<input type="submit" name="button_spellcheck" value="%s">'
-                % _('Check Spelling')) or ''
-
-        save_button_text = _('Save Changes')
-        cancel_button_text = _('Cancel')
-        
-        self.request.write("</p>")
-            
-        if self.cfg.page_license_enabled:
-            self.request.write('<p><em>', _(
-"""By hitting '''%(save_button_text)s''' you put your changes under the %(license_link)s.
-If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.""") % {
-                'save_button_text': save_button_text,
-                'cancel_button_text': cancel_button_text,
-                'license_link': wikiutil.getSysPage(self.request, self.cfg.page_license_page).link_to(self.request),
-            }, '</em></p>')
-
-        self.request.write('''
-<p>
-<input type="submit" name="button_save" value="%s">
-<input type="submit" name="button_preview" value="%s"> %s
-<input type="submit" name="button_cancel" value="%s">
-</p>
-<p>
-''' % (save_button_text, _('Preview'), button_spellcheck, cancel_button_text,))
-
-        if self.cfg.mail_smarthost:
+        if self.cfg.mail_enabled:
             self.request.write('''
-<input type="checkbox" name="trivial" value="1" %(checked)s>
-<label>%(label)s</label> ''' % {
+&nbsp;
+<input type="checkbox" name="trivial" id="chktrivial" value="1" %(checked)s>
+<label for="chktrivial">%(label)s</label> ''' % {
                 'checked': ('', 'checked')[form.get('trivial',['0'])[0] == '1'],
                 'label': _("Trivial change"),
                 })
 
         self.request.write('''
-<input type="checkbox" name="rstrip" value="1" %(checked)s>
-<label>%(label)s</label>
-</p> ''' % {
+&nbsp;
+<input type="checkbox" name="rstrip" id="chkrstrip" value="1" %(checked)s>
+<label for="chkrstrip">%(label)s</label>
+''' % {
             'checked': ('', 'checked')[form.get('rstrip',['0'])[0] == '1'],
             'label': _('Remove trailing whitespace from each line')
             })
+        self.request.write("</p>")
 
         badwords_re = None
         if preview is not None:
@@ -434,15 +412,19 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
         self.request.write("</form>")
         
         # QuickHelp originally by Georg Mischler <schorsch@lightingwiki.com>
-        self.request.write('<hr>\n<dl>' + _(""" Emphasis:: [[Verbatim('')]]''italics''[[Verbatim('')]]; [[Verbatim(''')]]'''bold'''[[Verbatim(''')]]; [[Verbatim(''''')]]'''''bold italics'''''[[Verbatim(''''')]]; [[Verbatim('')]]''mixed ''[[Verbatim(''')]]'''''bold'''[[Verbatim(''')]] and italics''[[Verbatim('')]]; [[Verbatim(----)]] horizontal rule.
- Headings:: [[Verbatim(=)]] Title 1 [[Verbatim(=)]]; [[Verbatim(==)]] Title 2 [[Verbatim(==)]]; [[Verbatim(===)]] Title 3 [[Verbatim(===)]];   [[Verbatim(====)]] Title 4 [[Verbatim(====)]]; [[Verbatim(=====)]] Title 5 [[Verbatim(=====)]].
- Lists:: space and one of: * bullets; 1., a., A., i., I. numbered items; 1.#n start numbering at n; space alone indents.
- Links:: [[Verbatim(JoinCapitalizedWords)]]; [[Verbatim(["brackets and double quotes"])]]; url; [url]; [url label].
- Tables:: || cell text |||| cell text spanning 2 columns ||;    no trailing white space allowed after tables or titles.""")
- + '</dl>')
+        markup = self.pi_format or self.request.cfg.default_markup
+        quickhelp = self.request.cfg.editor_quickhelp.get(markup, "")
+        if quickhelp:
+            self.request.write(self.request.formatter.div(1, id="editor-help"))
+            self.request.write(_(quickhelp))
+            self.request.write(self.request.formatter.div(0))
 
         if preview is not None:
-            self.send_page(self.request, content_id='preview', content_only=1,
+            if staytop:
+                content_id = 'previewbelow'
+            else:
+                content_id = 'preview'
+            self.send_page(self.request, content_id=content_id, content_only=1,
                            hilite_re=badwords_re)
 
         self.request.write(self.request.formatter.endContent())
@@ -464,25 +446,85 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
         page = backto and Page(self.request, backto) or self
         page.send_page(self.request, msg=_('Edit was cancelled.'))
 
+    def renamePage(self, newpagename, comment=None):
+        """
+        Rename the current version of the page (making a backup before deletion
+        and keeping the backups, logs and attachments).
+        
+        @param comment: Comment given by user
+        @rtype: unicode
+        @return: success flag, error message
+        """
+        _ = self._
+        if not newpagename:
+            return False, _("You can't rename to an empty pagename.")
+
+        newpage = PageEditor(self.request, newpagename)
+
+        pageexists_error = _("""'''A page with the name {{{'%s'}}} already exists.'''
+
+Try a different name.""") % (newpagename,)
+
+        # Check whether a page with the new name already exists
+        if newpage.exists(includeDeleted=1):
+            return False, pageexists_error
+        
+        # Get old page text
+        savetext = self.get_raw_body()
+
+        oldpath = self.getPagePath(check_create=0)
+        newpath = newpage.getPagePath(check_create=0)
+
+        # Rename page
+
+        # NOTE: might fail if another process created newpagename just
+        # NOW, while you read this comment. Rename is atomic for files -
+        # but for directories, rename will fail if the directory
+        # exists. We should have global edit-lock to avoid this.
+        # See http://docs.python.org/lib/os-file-dir.html
+        try:
+            os.rename(oldpath, newpath)
+            self.error = None
+            # Save page text with a comment about the old name
+            savetext = u"## page was renamed from %s\n%s" % (self.page_name, savetext)
+            newpage.saveText(savetext, 0, comment=comment)
+            return True, None
+        except OSError, err:
+            # Try to understand what happened. Maybe its better to check
+            # the error code, but I just reused the available code above...
+            if newpage.exists(includeDeleted=1):
+                return False, pageexists_error
+            else:
+                return False, _('Could not rename page because of file system error: %s.') % unicode(err)
+
     def deletePage(self, comment=None):
         """
         Delete the current version of the page (making a backup before deletion
         and keeping the backups, logs and attachments).
         
         @param comment: Comment given by user
+        @rtype: unicode
+        @return: success flag, error message
         """
-        # First save a final backup copy of the current page
-        # (recreating the page allows access to the backups again)
+        _ = self._
+        success = True
         try:
-            self.saveText(u"deleted\n", 0, comment=comment or u'')
-        except self.SaveError, msg:
+            # First save a final backup copy of the current page
+            # (recreating the page allows access to the backups again)
+            msg = self.saveText(u"deleted\n", 0, comment=comment or u'')
+            msg = msg.replace(
+                _("Thank you for your changes. Your attention to detail is appreciated."),
+                _('Page "%s" was successfully deleted!') % (self.page_name,))
+            # Then really delete it
+            try:
+                os.remove(self._text_filename())
+            except OSError, err:
+                if err.errno != errno.ENOENT:
+                    raise err
+        except self.SaveError, message:
             # XXX Error handling
-            pass
-        # Then really delete it
-        try:
-            os.remove(self._text_filename())
-        except OSError, er:
-            if er.errno != errno.ENOENT: raise er
+            success = False
+            msg = "SaveError has occured in PageEditor.deletePage. We need locking there."
         
         # reset page object
         self.reset()
@@ -505,6 +547,7 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
             key = formatter_name
             cache = caching.CacheEntry(self.request, arena, key)
             cache.remove()
+        return success, msg
 
     def _sendNotification(self, comment, emails, email_lang, revisions, trivial):
         """
@@ -566,8 +609,7 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
         @return: message, indicating success or errors.
         """
         _ = self._
-        subscribers = self.getSubscribers(self.request, return_users=1,
-                                          trivial=trivial)
+        subscribers = self.getSubscribers(self.request, return_users=1, trivial=trivial)
         if subscribers:
             # get a list of old revisions, and append a diff
             revisions = self.getRevList()
@@ -599,18 +641,19 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
         """
         # TODO: Allow addition of variables via wikiconfig or a global
         # wiki dict.
-        now = time.strftime("%Y-%m-%dT%H:%M:%SZ", util.datetime.tmtuple())
+        now = time.strftime("%Y-%m-%dT%H:%M:%SZ", timefuncs.tmtuple())
         user = self.request.user
         signature = user.signature()
         variables = {
             'PAGE': self.page_name,
             'TIME': "[[DateTime(%s)]]" % now,
             'DATE': "[[Date(%s)]]" % now,
+            'ME': user.name,
             'USERNAME': signature,
             'USER': "-- %s" % signature,
             'SIG': "-- %s [[DateTime(%s)]]" % (signature, now),
         }
-
+  
         if user.valid and user.name:
             if user.email:
                 variables['MAILTO'] = "[[MailTo(%s)]]" % user.email
@@ -619,12 +662,12 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
             userDictPage = user.name + "/MyDict"
             if self.request.dicts.has_dict(userDictPage):
                 variables.update(self.request.dicts.dict(userDictPage))
-                    
+                      
         # TODO: Use a more stream-lined re.sub algorithm
         for name in variables:
             text = text.replace('@%s@' % name, variables[name])
         return text
-
+  
     def normalizeText(self, text, **kw):
         """ Normalize text
 
@@ -662,30 +705,25 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
         Replace text of the user UserName/MoinEditorBackup with newtext.
 
         @param newtext: new text of the page
-        @keyword: none used currently
+        @param kw: no keyword args used currently
         @rtype: unicode
         @return: url of page backup or None
         """
         # Backup only if set to backup and user has a home page.
-        homepage = wikiutil.getHomePage(self.request)
-        if not homepage or not self.do_editor_backup:
+        homepage = wikiutil.getInterwikiHomePage(self.request)
+        if homepage is None or not self.do_editor_backup:
             return None
-        
+        if homepage[0] != 'Self': # user has a remote homepage
+            return None
+
         _ = self._
-        if config.allow_subpages:
-            delimiter = "/"
-        else:
-            delimiter = ""
         backuppage = PageEditor(self.request,
-                                homepage.page_name + delimiter + "MoinEditorBackup",
+                                homepage[1] + "/MoinEditorBackup",
                                 do_revision_backup=0)
-        if self.cfg.acl_enabled:
-            # We need All: at the end to prevent that the original page ACLs
-            # make it possible to see preview saves (that maybe were never
-            # really saved by the author).
-            intro = u"#acl %s:read,write,delete All:\n" % self.request.user.name
-        else:
-            intro = ""
+        # We need All: at the end to prevent that the original page ACLs
+        # make it possible to see preview saves (that maybe were never
+        # really saved by the author).
+        intro = u"#acl %s:read,write,delete All:\n" % self.request.user.name
         pagename = self.page_name
         date = self.request.user.getFormattedDateTime(time.time())
         intro += _('## backup of page "%(pagename)s" submitted %(date)s') % {
@@ -835,7 +873,6 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
 
         return mtime_usecs, rev
 
-
     def saveText(self, newtext, rev, **kw):
         """ Save new text for a page.
 
@@ -856,8 +893,7 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
         #!!! rev check is not enough since internal operations use "0"
 
         # expand variables, unless it's a template or form page
-        if not (wikiutil.isTemplatePage(self.request, self.page_name) or
-                wikiutil.isFormPage(self.request, self.page_name)):
+        if not wikiutil.isTemplatePage(self.request, self.page_name): 
             newtext = self._expand_variables(newtext)
 
         msg = ""
@@ -871,6 +907,22 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
             msg = _('You cannot save empty pages.')
             raise self.EmptyPage, msg
         elif rev != 0 and rev != self.current_rev():
+            # check if we already saved that page
+            other = False
+            pagelog = self.getPagePath('edit-log', use_underlay=0, isfile=1)
+            for line in editlog.EditLog(self.request, pagelog).reverse():
+                if int(line.rev)==int(rev):
+                    break
+                if not line.is_from_current_user(self.request):
+                    other = True
+                next_line = line
+            if next_line.is_from_current_user(self.request):
+                saved_page = Page(self.request, self.page_name,
+                                  rev=int(next_line.rev))
+                if newtext==saved_page.get_raw_body():
+                    msg = _("You already saved this page!")
+                    return msg
+                
             msg = _("""Sorry, someone else saved the page while you edited it.
 
 Please do the following: Use the back button of your browser, and cut&paste
@@ -889,7 +941,7 @@ delete the changes of the other person, which is excessively rude!''
         elif newtext == self.get_raw_body():
             msg = _('You did not change the page content, not saved!')
             raise self.Unchanged, msg
-        elif self.cfg.acl_enabled:
+        else:
             from wikiacl import parseACL
             # Get current ACL and compare to new ACL from newtext. If
             # they are not the sames, the user must have admin
@@ -919,7 +971,7 @@ delete the changes of the other person, which is excessively rude!''
             self.clean_acl_cache()
   
             # send notification mails
-            if self.request.cfg.mail_smarthost:
+            if self.request.cfg.mail_enabled:
                 msg = msg + self._notifySubscribers(comment, trivial)
           
             if self.request.cfg.lupy_search:
@@ -1028,15 +1080,13 @@ class PageLock:
                     ) % {'owner': owner, 'timestamp': timestamp, 'mins_valid': mins_valid}
             else:
                 # warn user about existing lock
-### WIKIMARKUP-TODO
+
                 result = 1, _(
-                    'This page was opened for editing or last previewed at %(timestamp)s by %(owner)s.[[BR]]\n'
-                    '\'\'\''
-                    'You should \'\'refrain from editing\'\' this page for at least another %(mins_valid)d minute(s),\n'
-                    'to avoid editing conflicts.'
-                    '\'\'\'[[BR]]\n'
-                    'To leave the editor, press the Cancel button.'
-                    ) % {'timestamp': timestamp, 'owner': owner, 'mins_valid': mins_valid}
+"""This page was opened for editing or last previewed at %(timestamp)s by %(owner)s.[[BR]]
+'''You should ''refrain from editing'' this page for at least another %(mins_valid)d minute(s),
+to avoid editing conflicts.'''[[BR]]
+To leave the editor, press the Cancel button.""") % {
+                    'timestamp': timestamp, 'owner': owner, 'mins_valid': mins_valid}
 
         return result
 
