@@ -85,7 +85,8 @@
         * fixed CSS for IE users
         * fix javascript for IE4
         * do a correct calculation of "today" using user's timezone
-
+    * 2.2:
+        * added template argument for specifying an edit template for new pages
 
     Usage:
         [[MonthCalendar(BasePage,year,month,monthoffset,monthoffset2,height6)]]
@@ -138,6 +139,10 @@ Anniversary Calendars: (no year data)
 This creates calendars of the format Yearly/MM-DD 
 By leaving out the year, you can set birthdays, and anniversaries in this 
 calendar and not have to re-enter each year.
+
+This creates a calendar which uses MonthCalendarTemplate for directly editing
+nonexisting day pages:
+[[MonthCalendar(,,,,,,MonthCalendarTemplate)]]
 """
 
 Dependencies = ['namespace','time']
@@ -153,88 +158,6 @@ import re, calendar, time
 # XXX change here ----------------vvvvvv
 calendar.setfirstweekday(calendar.MONDAY)
 
-infobox_js = r"""
-//
-// Bubblehelp infoboxes, (C) 2002 Klaus Knopper <infobox@knopper.net>
-// You can copy/modify and distribute this code under the conditions
-// of the GNU GENERAL PUBLIC LICENSE Version 2.
-//
-var IWIDTH=350  // Tip box width
-var ie4         // Are we using Internet Explorer Version 4?
-var ie5         // Are we using Internet Explorer Version 5 and up?
-var kon         // Are we using KDE Konqueror?
-var x,y,winW,winH  // Current help position and main window size
-var idiv=null   // Pointer to infodiv container
-
-function nsfix(){setTimeout("window.onresize = rebrowse", 2000);}
-
-function rebrowse(){window.location.reload();}
-
-function hascss(){ return gettip('infodiv')?true:false }
-
-function infoinit(){
- ie4=(document.all)?true:false;
- ie5=((ie4)&&((navigator.userAgent.indexOf('MSIE 5')>0)||(navigator.userAgent.indexOf('MSIE 6')>0)))?true:false;
- kon=(navigator.userAgent.indexOf('konqueror')>0)?true:false;
- x=0;y=0;winW=800;winH=600;
- idiv=null;
- document.onmousemove = mousemove;
- // Workaround for just another netscape bug: Fix browser confusion on resize
- // obviously conqueror has a similar problem :-(
- if(kon){ nsfix() }
-}
-
-function untip(){
- if(idiv) idiv.visibility="hidden";
- idiv=null;
-}
-
-function gettip(name){return (document.layers&&document.layers[name])?document.layers[name]:(document.all&&document.all[name]&&document.all[name].style)?document.all[name].style:document[name]?document[name]:(document.getElementById(name)?document.getElementById(name).style:0);}
-
-// Prepare tip boxes, but don't show them yet
-function maketip(name,title,text){
- if(hascss()) document.write('<div id="'+name+'" name="'+name+'" style="position:absolute; visibility:hidden; z-index:20; top:-999em; left:0px;"><table width='+IWIDTH+' class="tip"><tr><th class="tip">'+title+'</th></tr><tr><td class="tip">'+text+'</td></tr></table></div>\n');
-}
-
-function tip(name){
- if(hascss()){
-  if(idiv) untip();
-  idiv=gettip(name);
-  if(idiv){
-   winW=(window.innerWidth)? window.innerWidth+window.pageXOffset-16:document.body.offsetWidth-20;
-   winH=(window.innerHeight)?window.innerHeight+window.pageYOffset  :document.body.offsetHeight;
-   if(x<=0||y<=0){ // konqueror can't get mouse position
-    x=(winW-IWIDTH)/2+(window.pageXOffset?window.pageXOffset:0); y=(winH-50)/2+(window.pageYOffset?window.pageYOffset:0); // middle of window
-   }
-   showtip();
-  }
- }
-}
-
-function showtip(){
-  idiv.left=(((x+IWIDTH+10)<winW)?x+12:x-IWIDTH-5)+"px"; idiv.top=(((y+90)<winH)?y+12:y-90)+"px";
-  idiv.visibility="visible";
-//  window.status="idiv="+idiv+"X:"+(idiv.left?idiv.left:"NAN")+", Y:"+(idiv.top?idiv.top:"NAN")+", x:"+x+", y:"+y;
-}
-
-function mousemove(e){
- if(e)   {x=e.pageX?e.pageX:e.clientX?e.clientX:0; y=e.pageY?e.pageY:e.clientY?e.clientY:0;}
- else if(event) {x=event.clientX; y=event.clientY;}
- else {x=0; y=0;}
- if((ie4||ie5) && document.documentElement) // Workaround for scroll offset of IE
-  {
-    x+=document.documentElement.scrollLeft;
-    y+=document.documentElement.scrollTop;
-  }
- if(idiv) showtip();
-}
-
-// Initialize after loading the page
-window.onload=infoinit;
-
-// EOF infobox.js
-"""
-
 def cliprgb(r,g,b): # don't use 255!
     if r < 0:   r=0
     if r > 254: r=254
@@ -255,7 +178,7 @@ def yearmonthplusoffset(year, month, offset):
         year = year + 1
     return year, month
 
-def parseargs(args, defpagename, defyear, defmonth, defoffset, defoffset2, defheight6, defanniversary):
+def parseargs(args, defpagename, defyear, defmonth, defoffset, defoffset2, defheight6, defanniversary, deftemplate):
     strpagename = args.group('basepage')
     if strpagename:
         parmpagename = wikiutil.unquoteWikiname(strpagename)
@@ -300,7 +223,12 @@ def parseargs(args, defpagename, defyear, defmonth, defoffset, defoffset2, defhe
     else:
         parmanniversary = defanniversary
 
-    return parmpagename, parmyear, parmmonth, parmoffset, parmoffset2, parmheight6, parmanniversary
+    strtemplate = args.group('template')
+    if strtemplate:
+        parmtemplate = wikiutil.unquoteWikiname(strtemplate)
+    else:
+        parmtemplate = deftemplate
+    return parmpagename, parmyear, parmmonth, parmoffset, parmoffset2, parmheight6, parmanniversary, parmtemplate
         
 # FIXME:                          vvvvvv is there a better way for matching a pagename ?
 _arg_basepage = r'\s*(?P<basepage>[^, ]+)?\s*'
@@ -310,9 +238,10 @@ _arg_offset = r',\s*(?P<offset>[+-]?\d+)?\s*'
 _arg_offset2 = r',\s*(?P<offset2>[+-]?\d+)?\s*'
 _arg_height6 = r',\s*(?P<height6>[+-]?\d+)?\s*'
 _arg_anniversary =  r',\s*(?P<anniversary>[+-]?\d+)?\s*'
-_args_re_pattern = r'^(%s)?(%s)?(%s)?(%s)?(%s)?(%s)?(%s)?$' % \
+_arg_template = r',\s*(?P<template>[^, ]+)?\s*' # XXX see basepage comment
+_args_re_pattern = r'^(%s)?(%s)?(%s)?(%s)?(%s)?(%s)?(%s)?(%s)?$' % \
                      (_arg_basepage,_arg_year,_arg_month, \
-                      _arg_offset,_arg_offset2,_arg_height6,_arg_anniversary)
+                      _arg_offset,_arg_offset2,_arg_height6,_arg_anniversary,_arg_template)
 
 
 def execute(macro, text):
@@ -324,7 +253,7 @@ def execute(macro, text):
     if request.mode_getpagelinks:
         return ''
 
-    args_re=re.compile(_args_re_pattern)
+    args_re = re.compile(_args_re_pattern)
     
     currentyear, currentmonth, currentday, h, m, s, wd, yd, ds = request.user.getTime(time.time())
     thispage = formatter.page.page_name
@@ -336,22 +265,22 @@ def execute(macro, text):
             return ('<p><strong class="error">%s</strong></p>' % _('Invalid MonthCalendar calparms "%s"!')) % (text2,)
         else:
             has_calparms = 1 # yes!
-            cparmpagename, cparmyear, cparmmonth, cparmoffset, cparmoffset2, cparmheight6, cparmanniversary = \
-                parseargs(args2, thispage, currentyear, currentmonth, 0, 0, 0, 0)
+            cparmpagename, cparmyear, cparmmonth, cparmoffset, cparmoffset2, cparmheight6, cparmanniversary, cparmtemplate = \
+                parseargs(args2, thispage, currentyear, currentmonth, 0, 0, 0, 0, '')
     else:
         has_calparms = 0
 
     if text is None: # macro call without parameters
-        parmpagename, parmyear, parmmonth, parmoffset, parmoffset2, parmheight6, anniversary = \
-            [thispage], currentyear, currentmonth, 0, 0, 0, 0
+        parmpagename, parmyear, parmmonth, parmoffset, parmoffset2, parmheight6, anniversary, parmtemplate = \
+            [thispage], currentyear, currentmonth, 0, 0, 0, 0, ''
     else:
         # parse and check arguments
         args = args_re.match(text)
         if not args:
             return ('<p><strong class="error">%s</strong></p>' % _('Invalid MonthCalendar arguments "%s"!')) % (text,)
         else:
-            parmpagename, parmyear, parmmonth, parmoffset, parmoffset2, parmheight6, anniversary = \
-                parseargs(args, thispage, currentyear, currentmonth, 0, 0, 0, 0)
+            parmpagename, parmyear, parmmonth, parmoffset, parmoffset2, parmheight6, anniversary, parmtemplate = \
+                parseargs(args, thispage, currentyear, currentmonth, 0, 0, 0, 0, '')
 
     # does url have calendar params and is THIS the right calendar to modify (we can have multiple
     # calendars on the same page)?
@@ -361,6 +290,7 @@ def execute(macro, text):
     if has_calparms and cparmpagename == parmpagename:
         year,month = yearmonthplusoffset(parmyear, parmmonth, parmoffset + cparmoffset2)
         parmoffset2 = cparmoffset2
+        parmtemplate = cparmtemplate
     else:
         year,month = yearmonthplusoffset(parmyear, parmmonth, parmoffset)
 
@@ -380,11 +310,12 @@ def execute(macro, text):
     colorstep = 85
     p = Page(request, thispage)
     qpagenames = '*'.join(map(wikiutil.quoteWikinameURL, parmpagename))
-    querystr = "calparms=%%s,%d,%d,%d,%%d" % (parmyear, parmmonth, parmoffset)
-    prevlink  = p.url(request, querystr % (qpagenames, parmoffset2 - 1))
-    nextlink  = p.url(request, querystr % (qpagenames, parmoffset2 + 1))
-    prevylink = p.url(request, querystr % (qpagenames, parmoffset2 - 12))
-    nextylink = p.url(request, querystr % (qpagenames, parmoffset2 + 12))
+    qtemplate = wikiutil.quoteWikinameURL(parmtemplate)
+    querystr = "calparms=%%s,%d,%d,%d,%%d,%%s" % (parmyear, parmmonth, parmoffset)
+    prevlink  = p.url(request, querystr % (qpagenames, parmoffset2 - 1, qtemplate), 0)
+    nextlink  = p.url(request, querystr % (qpagenames, parmoffset2 + 1, qtemplate), 0)
+    prevylink = p.url(request, querystr % (qpagenames, parmoffset2 - 12, qtemplate), 0)
+    nextylink = p.url(request, querystr % (qpagenames, parmoffset2 + 12, qtemplate), 0)
     prevmonth = formatter.url(1, prevlink, 'cal-link') + '&lt;' + formatter.url(0)
     nextmonth = formatter.url(1, nextlink, 'cal-link') + '&gt;' + formatter.url(0)
     prevyear  = formatter.url(1, prevylink, 'cal-link') + '&lt;&lt;' + formatter.url(0)
@@ -460,24 +391,32 @@ def execute(macro, text):
                 daypage = Page(request, link)
                 if daypage.exists():
                     csslink = "cal-usedday"
+                    query = {}
                     r, g, b, u = (255, 0, 0, 1)
                     daycontent = daypage.get_raw_body()
                     header1_re = re.compile(r'^\s*=\s(.*)\s=$', re.MULTILINE) # re.UNICODE
                     titletext = []
                     for match in header1_re.finditer(daycontent):
                         if match:
-                            titletext.append(match.group(1))
+                            title = match.group(1)
+                            title = wikiutil.escape(title).replace("'","\\'")
+                            titletext.append(title)
                     tipname = link
                     tiptitle = link
                     tiptext = '<br>'.join(titletext)
                     maketip_js.append("maketip('%s','%s','%s');" % (tipname, tiptitle, tiptext))
-                    onmouse = '''onMouseOver="tip('%s')" onMouseOut="untip()"''' % tipname
+                    onmouse = {'onMouseOver': "tip('%s')" % tipname,
+                               'onMouseOut':  "untip()"}
                 else:
                     csslink = "cal-emptyday"
+                    if parmtemplate:
+                        query = {'action': 'edit', 'template': parmtemplate}
+                    else:
+                        query = {}
                     r, g, b, u = (255, 255, 255, 0)
                     if wkday in wkend:
                         csslink = "cal-weekend"
-                    onmouse = ''
+                    onmouse = {}
                 for otherpage in parmpagename[1:]:
                     otherlink = "%s/%4d-%02d-%02d" % (otherpage, year, month, day)
                     otherdaypage = Page(request, otherlink)
@@ -489,7 +428,7 @@ def execute(macro, text):
                             r, g, b = (r, g+colorstep, b)
                 r, g, b = cliprgb(r, g, b)
                 style = 'background-color:#%02x%02x%02x' % (r, g, b)
-                fmtlink = formatter.url(1, daypage.url(request), csslink, attrs=onmouse) + str(day) + formatter.url(0)
+                fmtlink = formatter.url(1, daypage.url(request, query), csslink, **onmouse) + str(day) + formatter.url(0)
                 if day == currentday and month == currentmonth and year == currentyear:
                     cssday = "cal-today"
                     fmtlink = "<b>%s</b>" % fmtlink # for browser with CSS probs
@@ -502,11 +441,7 @@ def execute(macro, text):
     restable = restable % (restr1, restr2, "".join(restrn))
 
     result = """\
-<script language="JavaScript" type="text/javascript">
-<!--
-%s
-// -->
-</script>
+<script language="JavaScript" type="text/javascript" src="%s/common/js/infobox.js"></script>
 <div id="infodiv" style="position:absolute; visibility:hidden; z-index:20; top:-999em; left:0px;"></div>
 <script language="JavaScript" type="text/javascript">
 <!--
@@ -514,7 +449,7 @@ def execute(macro, text):
 // -->
 </script>
 %s
-""" % (infobox_js, "\n".join(maketip_js), restable)
+""" % (request.cfg.url_prefix, "\n".join(maketip_js), restable)
     return formatter.rawHTML(result)
 
 # EOF
